﻿using iFileProxy.Config;
using iFileProxy.Helpers;
using iFileProxy.Models;
using iFileProxy.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.StaticFiles;

namespace iFileProxy.Controllers
{
    public class iProxyController : ControllerBase
    {
        static readonly TaskManager taskManager = new ();
        static Dictionary<string, Dictionary<string, uint>> IPAccessCountDict = [];

        [HttpPost]
        [Route("/AddOfflineTask")]
        public ActionResult<CommonRsp> AddOfflineTask()
        {   
            return taskManager.AddTask(HttpContext) switch
            {
                TaskAddState.Success => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.Success, message = "succ" }),
                TaskAddState.Fail => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.Fail, message = "unkown error!" }),
                TaskAddState.ErrUrlRepeat => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrUrlRepeat, message = "此url已经在任务队列中,请勿重复提交" }),
                TaskAddState.ErrTaskIdRepeat => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrTaskIdRepeat, message = "TaskIdRepeat!!!" }),
                TaskAddState.ErrUrlInvalid => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrUrlInvalid, message = "非法Url" }),
                TaskAddState.ErrDbFail => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrDbFail, message = "数据库数据提交失败!" }),
                TaskAddState.ErrMaxParallelTasksLimit => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrMaxParallelTasksLimit, message = "服务端并行数量达到上限!" }),
                TaskAddState.ErrFileSizeLimit => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrFileSizeLimit, message = "指定的文件大小超过系统最大限制!" }),
                TaskAddState.ErrFileNameForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrFileNameForbidden, message = "文件名非法!" }),
                TaskAddState.ErrIPForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrIPForbidden, message = "请求次数超过限制!" }),
                TaskAddState.ErrTargetHostForbidden => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrTargetHostForbidden, message = "目标主机不在服务白名单内!" }),
                TaskAddState.ErrGetFileInfo => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.ErrGetFileInfo, message = "目标文件信息获取失败!" }),

                _ => (ActionResult<CommonRsp>)Ok(new CommonRsp() { retcode = (int)TaskAddState.Success, message = "succ default" }),
            };
        }

        [HttpPost]
        [HttpGet]
        [Route("/GetMyTasks")]
        public ActionResult<CommonRsp> GetMyTasks()
        {
            return Ok(new CommonRsp() { retcode = 0,  data = taskManager.GetTaskListByIpAddr(HttpContext),message = "succ" });
        }

        [HttpGet]
        [HttpPost]
        [Route("/Download/{taskID}")]
        public async Task<IActionResult> DownloadFile(string taskID)
        {
            string fileName = "";
            var d = taskManager.GetTaskListByIpAddr(HttpContext);
            if (d.Where(x => x.TaskId == taskID).Any())
            {
                fileName = d.Where(x => x.TaskId == taskID).FirstOrDefault()?.FileName;
                var filePath = Path.Combine(AppConfig.GetCurrConfig().DownloadOptions.SavePath, fileName);
                if (!System.IO.File.Exists(filePath))
                {
                    return Ok(new CommonRsp() { message = "file not exists", retcode = 1 });
                }
                // 获取文件的 MIME 类型
                var provider = new FileExtensionContentTypeProvider();
                string contentType;
                if (!provider.TryGetContentType(filePath, out contentType))
                {
                    contentType = "application/octet-stream"; // 默认 MIME 类型
                }

                // 读取文件内容
                var fileContents = await System.IO.File.ReadAllBytesAsync(filePath);

                // 返回文件内容
                return File(fileContents, contentType, Path.GetFileName(filePath));
            }
            return Ok(new CommonRsp() { message = "task_id not exists", retcode = -1 });

        }
    }
}
