﻿using iFileProxy.Models;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace iFileProxy.Helpers
{
    public class MasterHelper
    {
        /// <summary>
        /// 检测链接是否为合法的网址格式
        /// </summary>
        /// <param name="uri">待检测的链接</param>
        /// <returns></returns>
        public static bool CheckUrlIsValid(string? uri)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(uri))
                    return false;

                var regex = @"(http://)?([\w-]+\.)+[\w-]+(/[\w- ./?%&=]*)?";
                Regex re = new Regex(regex);
                return re.IsMatch(uri);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
            return false;
        }

        /// <summary>
        /// 获取一个给定的范围随机数
        /// </summary>
        /// <param name="minValue">最小值</param>
        /// <param name="maxValue">最大值</param>
        /// <returns></returns>
        public static int GenerateRandomNumber(int minValue, int maxValue)
        {
            Random random = new Random(); // 创建一个新的Random实例
            int randomNumber = random.Next(minValue, maxValue + 1); // 生成位于[minValue, maxValue]范围内的随机数
            return randomNumber;
        }

        public static string? GetClientIPAddr(HttpContext c)
        {
            // 尝试从 X-Forwarded-For 请求头获取客户端IP地址
            string? clientIp = c.Request.Headers["X-Forwarded-For"].FirstOrDefault();
            // 如果 X-Forwarded-For 头不存在，回退到 RemoteIpAddress
            if (string.IsNullOrEmpty(clientIp))
            {
                clientIp = c.Connection.RemoteIpAddress?.ToString();
            }
            return clientIp;
        }

        public static string GetFileHash(string fileName, FileHashAlgorithm algorithm)
        {
            byte[] hash = [];
            if (File.Exists(fileName))
            {
                hash = algorithm switch
                {
                    FileHashAlgorithm.MD5 => MD5.HashData(File.ReadAllBytes(fileName)),
                    FileHashAlgorithm.SHA1 => SHA1.HashData(File.ReadAllBytes(fileName)),
                    FileHashAlgorithm.SHA256 => SHA256.HashData(File.ReadAllBytes(fileName)),
                    FileHashAlgorithm.SHA384 => SHA384.HashData(File.ReadAllBytes(fileName)),
                    FileHashAlgorithm.SHA512 => SHA512.HashData(File.ReadAllBytes(fileName)),
                    _ => MD5.HashData(File.ReadAllBytes(fileName)),
                };
                StringBuilder sBuilder = new StringBuilder();
                for (int i = 0; i < hash.Length; i++)
                {
                    sBuilder.Append(hash[i].ToString("x2"));
                }
                return sBuilder.ToString();
            }
            return null;
        }



    }
}
