
using iFileProxy.Config;
using iFileProxy.Middleware;
using Serilog;

namespace iFileProxy
{
    public class Program
    {
        public static void Main(string[] args)
        {
            SerilogConfig.CreateLogger();
            Serilog.ILogger logger = Log.Logger.ForContext<Program>();

            var builder = WebApplication.CreateBuilder(args);

            // Add services to the container.

            builder.Services.AddControllers();
            // Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
            builder.Services.AddEndpointsApiExplorer();
            builder.Services.AddSwaggerGen();

            builder.Host.UseSerilog(logger: Log.Logger);

            var app = builder.Build();

            app.UseSerilogRequestLogging(options =>
            {
                options.EnrichDiagnosticContext = (diagCtx, httpCtx) =>
                {
                    diagCtx.Set("ClientIp", httpCtx.Connection.RemoteIpAddress?.ToString());
                    diagCtx.Set("contentType", httpCtx.Request.ContentType);
                    diagCtx.Set("queryString", httpCtx.Request.QueryString);
                };
            });

            // Configure the HTTP request pipeline.
            if (app.Environment.IsDevelopment())
            {
                app.UseSwagger();
                app.UseSwaggerUI();
            }

            app.UseHttpsRedirection();

            // Ĭļѡ
            var defaultFilesOptions = new DefaultFilesOptions();
            defaultFilesOptions.DefaultFileNames.Clear(); // Ĭб
            defaultFilesOptions.DefaultFileNames.Add("index.html"); // ԶĬļ

            app.UseDefaultFiles(defaultFilesOptions);

            app.UseStaticFiles();

            app.UseAuthorization();

            var IPAccessCountDict = new Dictionary<string, Dictionary<string, uint>>();
            var dailyRequestLimitPerIP = AppConfig.GetCurrConfig().SecurityOptions.DailyRequestLimitPerIP;
            app.UseMiddleware<IPAccessLimitMiddleware>(IPAccessCountDict, dailyRequestLimitPerIP);

            app.UseMiddleware<ErrorHandlerMiddleware>(); // м

            app.MapControllers();

            app.Run();
        }
    }
}
